!*  <<< SUBGUTY >>> --------------------------------------------------
!
!=====================================================================
character *(*)FUNCTION chgrtfn(FNAME, ext1, ext2)
  character FNAME*(*), EXT1*(*), ext2*(*) 
  character  extend*70, oext*5
!
! chgrtfn:
!      Add characters to end of the file name.

! Arguments:
!         INPUT/
!           FNAME   : FILE NAME TO BE CHANGED
!           ext1    : CHARACTERS FOLLOWING THE END OF "_" TO BE CHANGED
!                     IN THE FILE NAME
!           ext2    : CHARACTERS TO BE ADDED TO THE END OF FILE NAME
!         OUTPUT/
!           chgrtfn : FILE NAME ADDED NEW CHARACTERS WITH EXTENSION
!
!--
!     AUTHOR: H.Shimizu 2007/10/9
!----------------------------------------------------------------------

  leng = len_trim(fname)
  do i = leng, 1, -1
     if(fname(i:i)=='.') exit
  end do
  oext='fits'
  if ( i>1 ) then
     oext = fname(i+1:leng)
     leng=i-1 ! filename-length without extension
  end if

  do k = i, 1, -1
     if(fname(k:k)=='_') exit
  end do
  ip=k   ! position of under-score
  
  if (ip==0) then
     chgrtfn = fname(1:leng) //'_'// ext2
  else
     if ( fname(k+1:leng)==ext1 )  then
        chgrtfn = fname(1:k-1) //'_'// ext2
     else
        chgrtfn = fname(1:leng) //'_'// ext2
     end if
  end if

  chgrtfn = extend(chgrtfn,oext)
  
  return
  
END FUNCTION chgrtfn
!=====================================================================

!*EXTEND   -- add or change the file name extension
!+
character *(*)FUNCTION EXTEND(FNAME, EXT)
  character FNAME*(*), EXT*(*)
!
! EXTEND:
!      Replace the file name extension with another one.

! Arguments:
!         INPUT/
!           FNAME  : FILE NAME (character STRING) TO BE CHANGED
!           EXT    : NEW FILE-NAME EXTENSION
!         OUTPUT/
!           FNAME  : FILE NAME (character STRING) WITH NEW EXTENSION
!
!--
!     AUTHOR: M. Hamabe
!     UPDATE: H.Shimizu 2007/10/9
!----------------------------------------------------------------------
  
  leng = len_trim(fname) ! l = index(fname,' ')
  do i = leng, 1, -1
     if(fname(i:i)=='.') exit
  end do
  
  if (i .eq. 0) then
     extend = (fname(1:leng) // '.') // ext
  else
     extend = fname(1:i) // ext
  end if
  
  return

end FUNCTION EXTEND

!======================================================================

!*TOUPER   -- change lower-case letters to upper-case letters in a string
!+
character*(*) FUNCTION TOUPPER(string)
  character STRING*(*)

! TOUPPER:
! Change lower-case letters to upper-case letters in a string
!
! Arguments:
!         INPUT/
!           STRING  : character STRING TO BE CONVERTED
!         OUTPUT/
!           TOUPPER : character STRING (RESULT)
!
! Notes:
!         ASCII code is assumed.
!       
!--
! Author:   M. Hamabe (2007-10-18)
!     
!----------------------------------------------------------------------
  nlen=len_trim(string)
  toupper=string
  do i=1, nlen
     idum=ichar(toupper(i:i))
     if ( idum.gt.96 .and. idum.lt.123 ) then
        toupper(i:i)=char(idum-32)
     end if
  end do

return

end FUNCTION TOUPPER

!======================================================================

!*TOLOWER   -- change upper-case letters to lower-case letters in a string
!+
character*(*) FUNCTION TOLOWER(string)
  character STRING*(*)

! TOLOWERL:
!     Change upper-case letters to lower-case letters in a string
!
! Arguments:
!         INPUT/
!           STRING  : character STRING TO BE CONVERTED
!         OUTPUT/
!           TOLOWER : character STRING (RESULT)
!
! Notes:
!         ASCII code is assumed.
!
!--
! Author:   M. Hamabe (2007-10-18)
!     
!----------------------------------------------------------------------

  nlen=len_trim(string)
  tolower=string

  do i=1, nlen
     idum=ichar(tolower(i:i))
     if ( idum.gt.64 .and. idum.lt.91 ) then
        tolower(i:i)=char(idum+32)
     end if
  end do

return

end FUNCTION TOLOWER


!======================================================================

!*GETDATE  -- get date in 'yyyymmdd' format.
!+
SUBROUTINE GETDATE(CDATE)

  integer   :: val(8)
  character :: CDATE*8
  
! GETDATE:
!         Get date in 'YYYYMMDD' format.
!
! Arguments:
!         OUTPUT/
!           CDATE  : String containing the current date
!
! Other subroutines called: 
!         (S) DATE_AND_TIME
!
!--
!     AUTHOR: M.Hamabe  2007-09-22
! ---------------------------------------------------------------------
  
  CALL DATE_AND_TIME(VALUES=val)

  write(cdate(1:4),'(i4)') val(1)
  write(cdate(5:6),'(bz,i2)') val(2)
  if ( val(2)<10 ) cdate(5:5)='0'  
  write(cdate(7:8),'(i2)') val(3)
  if ( val(3)<10 ) cdate(7:7)='0'  
  
  return

end SUBROUTINE GETDATE

!======================================================================

!*ARCTAN   -- compute arctan (in degree).
!+
SUBROUTINE ARCTAN(X,Y,THETA,irc)
  
! ARCTAN:
!         COMPUTE ARCTAN (FOR ARGUMENT).
!
! Arguments:
!         INPUT/
!           X      : X-COORDINATE
!           Y      : Y-COORDINATE
!         OUTPUT/
!           THETA  : COMPUTED ANGLE ( IN DEGREE )
!           irc    : return CODE ( 0 : NORMAL END )
!
! Note:
!                  THETA= ARCTAN(Y/X)       0 <= THETA < 360 DEGREE
!
!--
!     AUTHOR: S.ICHIKAWA  1985-12-19
!     UPDATE: 1986-06-15 S.ICHIKAWA   (CORRECTION)
! ---------------------------------------------------------------------
  
  XX=ABS(X)
  YY=ABS(Y)
  ZZ=1.0E-20
  
  if (XX<ZZ.and.YY<ZZ) then
     irc=1000
     return
  else if (XX<ZZ.and.Y>0.0) then
     THETA= 90.0
  else if (XX<ZZ.and.Y<0.0) then
     THETA=270.0
  else if (YY<ZZ.and.X>0.0) then
     THETA=  0.0
  else if (YY<ZZ.and.X<0.0) then
     THETA=180.0
  else
     T=ATAN2(Y,X)
     THETA=57.29577951*T
     if (THETA<0.0) THETA=THETA+360.0
  end if

  irc=0

  return
END subroutine ARCTAN
!======================================================================

!*MEPS     -- compute the machine epsilon.
!+
SUBROUTINE MEPS (EPS4,EPS8)
  REAL*8 EPS8
  
! MEPS:
!         Compute the machine epsilon.
!
! Arguments:
!         OUTPUT/
!           EPS4   : Single-precision machine epsilon (REAL*4)
!           EPS8   : Double-precision machine epsilon (REAL*8)
!
!--
!     AUTHOR: S.OKAMURA 1985-05-10
!     UPDATE: M.Hamabe & Tsutomu Aoki  1990-12-03 (for SPARC)
!---------------------------------------------------------------------
  
  REAL*8 DUM8

!     *** COMPUTE SINGLE-PRECISION MACHINE EPSILON ***
  dum =1.5
  EPS4=1.0
  do while ( DUM > 1.0 )
     EPS4=0.5*EPS4
     DUM=EPS4+1.0
  end do

!     *** COMPUTE DOUBLE-PRECISION MACHINE EPSILON ***
  EPS8=1.0D0
  dum8=1.5D0
  do while ( DUM8 > 1.0D0 )
     EPS8=0.5D0*EPS8
     DUM8=EPS8+1.0D0
  end do

  return

END subroutine MEPS
