!* <<< SUBPHOTO >>> ---------------------------------------------------
!+
!	Checked by M. Hamabe	1990-12-03 	(for SPARC)
!				(only compiler-errors)
!---------------------------------------------------------------------

!=====================================================================

!*CINTEG   -- compute an integrated value in a circular region
!+
SUBROUTINE CINTEG (G,iext,jext,ci,cj,r,mode,ksub,fact,order,&
  &                val,pix,ax,ay,az,kext,irc)
  real(4)   :: G(iext,jext), ax(kext), ay(kext), az(kext)
  integer   :: order
!
! CINTEG:
!         compute an integrated value in a circular region.
!
! Arguments:
!         INPUT/
!           G      : Matrix
!           iext   : i-dimension of the matrix
!           jext   : j-dimension of the matrix
!           CI     : i-center    of the circular region of G
!           CJ     : j-center    of the circular region of G
!           R      : Radius of the circular region (pixel)
!           MODE   : Sub-pixel using           0 NO
!                                              1 YES
!           KSUB   : Background subtraction    0 NO
!                                              1 YES
!           fact   : Background region   RB=fact*R   1.5 =< fact =< 5.0
!           ord    : Background fitting order          0 =< order =< 5
!           KEXT   : Dimension of workarea
!         OUTPUT/
!           val    : Integration value
!           pix    : pixel number in the region
!           irc    : Return code      0    Normal end
!                                   11-14  Out of matrix
!                                   21-24  Out of matrix (fitting area)
!                                   1000   error of fact
!                                   3000   Fitting error
!                                   5000   Buffer overflow
!         WORKAREA/
!           AX AY AZ
!
! Other subroutines called:
!         (A) PLYNM2/ PLANE
!
! Note:
!         Sub-pixel is  1/5 * 1/5
!
!--
!     AUTHOR: S.ICHIKAWA 1985-04-04
!     UPDATE: S.ICHIKAWA 1985-10-20
!     UPDATE: S.ICHIKAWA 1985-11-12 FOR BACKGROUND-SUBTRACTION
!     UPDATE: S.ICHIKAWA 1986-03-31 FOR PARTIAL INTEGRATION
!     UPDATE: M.HAMABE   1987-11-27 BUG CORRECTION
!     UPDATE: M.HAMABE   1991-09-24 USE PLANE
! ---------------------------------------------------------------------

  logical  :: subtract
  real(8)  :: SI, SP, BI, COEF(21), BX, BY, BZ
  integer  :: ord

  subtract=.false.
  if ( ksub==1 ) subtract=.true.

  ord=order
  irc=0
  
  I1=CI-2.0-R
  I2=CI+2.0+R
  J1=CJ-2.0-R
  J2=CJ+2.0+R
  if ( I1<1 )  then
     I1=   1
     irc=11
  end if
  if ( I2>iext )  then
     I2=iext
     irc=12
  end if
  if ( J1<1 )  then
     J1=   1
     irc=13
  end if
  if ( J2>jext )  then
     J2=jext
     irc=14
  end if

  SP=0.0D0
  SI=0.0D0
  
  if ( subtract ) then ! background will be corrected
     if ( fact<0.5 .or. fact>5.0 ) then
        irc=1000
        go to 1000
     end if
     RB=R*fact   ! outer radius of annulus
     IB1=CI-2.0-RB
     IB2=CI+2.0+RB
     JB1=CJ-2.0-RB
     JB2=CJ+2.0+RB
     if ( IB1<1 )  then
        IB1=   1
        irc=21
     end if
     if ( IB2>iext )  then
        IB2=iext
        irc=22
     end if
     if ( JB1<1 )  then
        JB1=   1
        irc=23
     end if
     if ( JB2>jext )  then
        JB2=jext
        irc=24
     end if
     NO=0
     do J=JB1,JB2
        Y=real(J)-CJ
        do I=IB1,IB2
           X=real(I)-CI
           R1=sqrt(X*X+Y*Y)
           if ( R1<=RB .and. R1>R ) then
              NO=NO+1
              if ( NO>KEXT ) then
                 irc=5000
                 go to 1000
              end if
              AX(NO)=X
              AY(NO)=Y
              AZ(NO)=G(I,J)
           end if
        end do
     end do
!----------------------------------------------------------     
!     if ( NO<21) then   ! number of points is too small 
!        irc=3000
!        go to 1000
!     end if
!     
!     call PLANE(AX,AY,AZ,NO,ord,COEF,SD)
!----------------------------------------------------------     

     if ( NO>=21) then 
        call PLANE(AX,AY,AZ,NO,ord,COEF,SD)
     else
        print *,'Warning: background is assumed to be constant'
        irc=3000
        ord=0
        sum=0.0
        do ii=1, no
           sum=sum+az(ii)
        end do
        coef(1)=sum/real(no)
     end if
     
!----------------------------------------------------------     
     BI=0.0D0
     if ( MODE==1 ) then
        do J=J1,J2
           Y=real(J)-CJ
           do I=I1,I2
              X=real(I)-CI
              R1=sqrt(X*X+Y*Y)-1.0
              R2=R1+2.0
              if ( R2<=R ) then
                 SP=SP+1.0D0
                 SI=SI+G(I,J)
                 BX=X
                 BY=Y
                 call PLYNM2(BX,BY,COEF,ord,BZ)
                 BI=BI+BZ
              else if ( R1<=R ) then
                 do JJ=-2,2
                    YY=real(JJ)*0.2+Y
                    do II=-2,2
                       XX=real(II)*0.2+X
                       RR=sqrt(XX*XX+YY*YY)
                       if ( RR<=R ) then
                          SP=SP+4.0D-2
                          SI=SI+G(I,J)*4.0D-2
                          BX=XX
                          BY=YY
                          call PLYNM2(BX,BY,COEF,ord,BZ)
                          BI=BI+BZ*4.0D-2
                       end if
                    end do
                 end do
              end if
           end do
        end do
     else if ( MODE==0 ) then
        do J=J1,J2
           Y=real(J)-CJ
           do I=I1,I2
              X=real(I)-CI
              RR=sqrt(X*X+Y*Y)
              if ( RR<=R ) then
                 SP=SP+1.0D0
                 SI=SI+G(I,J)
                 BX=X
                 BY=Y
                 call PLYNM2(BX,BY,COEF,ord,BZ)
                 BI=BI+BZ
              end if
           end do
        end do
     end if
     pix=SP
     val=SI-BI
  else   ! no background correction
     if ( MODE==1 ) then
        do J=J1,J2
           Y=real(J)-CJ
           do I=I1,I2
              X=real(I)-CI
              R1=sqrt(X*X+Y*Y)-1.0
              R2=R1+2.0
              if ( R2<=R ) then
                 SP=SP+1.0D0
                 SI=SI+G(I,J)
              else if ( R1<=R ) then
                 do JJ=-2,2
                    YY=real(JJ)*0.2+Y
                    do II=-2,2
                       XX=real(II)*0.2+X
                       RR=sqrt(XX*XX+YY*YY)
                       if ( RR<=R ) then
                          SP=SP+4.0D-2
                          SI=SI+G(I,J)*4.0D-2
                       end if
                    end DO
                 end do
              end if
           end do
        end do
     else if ( MODE==0 ) then
        do J=J1,J2
           Y=real(J)-CJ
           do I=I1,I2
              X=real(I)-CI
              RR=sqrt(X*X+Y*Y)
              if ( RR<=R ) then
                 SP=SP+1.0D0
                 SI=SI+G(I,J)
              end if
           end do
        end do
     end if
     pix=SP
     val=SI
   end if

 1000 continue
   
   return

 END SUBROUTINE CINTEG
